/*************************************************************************\
*   				Xcard 1.2				  *
*									  *
* Copyright 1991, Kenneth C. Nelson			 	          *
*									  *
* Rights       : I, Ken Nelson own Xcard.  You may compile, modify, and   *
*                use Xcard.  You may not sell Xcard or anything derived   *
*		 from Xcard in binary or source for more than reasonable  *
*                media costs.                                             *
*                                                                         *
*		 Xcard uses a dynamic object library	   	          *
*		 that I don't own.  See the subdirectory dynobj for  	  *
*		 restrictions on its use.				  *
*								          *
*                Please feel free to modify Xcard. See Todo for details.  *
*									  *
\*************************************************************************/


/*
 * search.c - adds the search popup, and provides routines for getting the
 *	      search text.
 */
 
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Form.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h>



#include "xwidgets.h"
#include "xmisc.h"
#include "cards.h"
#include "search.h"


Widget searchPopup;
Widget searchTextWidget;
Widget whereRadioGroup, directionRadioGroup;




/*
 * CBcancelSearch() - pops down the search form.
 *
 */

/*ARGSUSED*/
void CBcancelSearch(w,call_data,client_data)
 Widget		w;
 XtPointer	call_data;
 XtPointer	client_data;
{
  XtPopdown(searchPopup);
}





/*
 * CBdoSearch() - callback that invokes the search.
 */
 

/*ARGSUSED*/
void CBdoSearch(w,call_data,client_data)
 Widget		w;
 XtPointer	call_data;
 XtPointer	client_data;
{
  if (doSearch(searchText()))
  {
    highlightHeader(currentCard());    
  }
  else
  {
    popupDismissableMessage(w,"\n\n  Search String Not Found   \n\n");
  }
  XtPopdown(searchPopup);
}





void setSearchDirection(direction)
  int direction;
{
  if (direction == FORWARD) 
  {
    XawToggleSetCurrent(directionRadioGroup,"goForward");
  }
  else if (direction == BACKWARD)
  {
    XawToggleSetCurrent(directionRadioGroup,"goBack");
  }
  
}


int searchDirection()
{
   char *text = XawToggleGetCurrent(directionRadioGroup);

   if (text != NULL)
   {
     if (strcmp(text,"goBack") == 0)
     {
       return BACKWARD;
     }
     else
     {
       return FORWARD;
     }
   }
   return FORWARD;
}


void setWhereToSearch(where)
  int where;
{
  if (where == IN_HEADER)
  {
    XawToggleSetCurrent(whereRadioGroup,"inHeader");
  }
  else if (where == IN_BODY)
  {
    XawToggleSetCurrent(whereRadioGroup,"inBody");
  }
  
}


int searchWhere()
{
  char *text = XawToggleGetCurrent(whereRadioGroup);

  if (text != NULL)
  {
    if (strcmp(text,"inHeader") == 0)
    {
      return IN_HEADER;
    }
    else
    {
      return IN_BODY;
    }
  }
  return IN_HEADER;
}




/*
 * addSearchPopup() - adds the buttons and text fields used to search forward
 *			and backwards in headers or in card bodies.
 *
 */


void  addSearchPopup(parent)
  Widget parent;
{
  Widget form,label,cancelButton,searchButton;
  Widget goBack, inBody;

 /*
  * An aciiTextWidget translation that disables carriage returns etc...
  *
  */

  static char oneLineTrans[] =  " Ctrl<Key>M : beginning-of-line()\n \
				 <Key>Return: beginning-of-line() ";
  XtTranslations  oneLineEditor;
  

   searchPopup = XtCreatePopupShell(
				    "searchPopup",
				    transientShellWidgetClass,
				    toplevel,
				    NULL,
				    0
				   );


   form = XtVaCreateManagedWidget("searchForm",
				  formWidgetClass,
				  searchPopup,
				  NULL
				 );

   label = XtVaCreateManagedWidget(
				   "searchLabel",
				   labelWidgetClass,
				   form,
				   XtNborderWidth,0,
				   XtNlabel,"Search For: ",
				   NULL
				 );
  				   
   searchTextWidget = XtVaCreateManagedWidget(
					      "searchTextWidget",
					      asciiTextWidgetClass,
					      form,
					      XtNfromHoriz,label,
					      XtNwidth,400,
					      XtNeditType,XawtextEdit,
					      NULL
					     );

   oneLineEditor = XtParseTranslationTable(oneLineTrans);
   XtOverrideTranslations(searchTextWidget,oneLineEditor);


   whereRadioGroup = XtVaCreateManagedWidget(
				    "inHeader",
				    toggleWidgetClass,
				    form,
				    XtNfromVert,label,
				    XtNlabel,"In Header",
				    XtNstate,TRUE,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

				   
   inBody = XtVaCreateManagedWidget(
				    "inBody",
				    toggleWidgetClass,
				    form,
				    XtNfromHoriz,whereRadioGroup,
				    XtNfromVert,label,
				    XtNlabel,"In Body",
				    XtNstate,FALSE,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

   XawToggleChangeRadioGroup(inBody,whereRadioGroup);
   XawToggleChangeRadioGroup(whereRadioGroup,inBody);

   directionRadioGroup = XtVaCreateManagedWidget(
				    "goBack",
				    toggleWidgetClass,
				    form,
				    XtNfromVert,whereRadioGroup,
				    XtNlabel,"Backward",
				    XtNvalue,TRUE,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

   goBack  = XtVaCreateManagedWidget(
				    "goForward",
				    toggleWidgetClass,
				    form,
				    XtNfromHoriz,directionRadioGroup,
				    XtNfromVert,whereRadioGroup,
				    XtNlabel,"Forward",
				    XtNvalue,FALSE,
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );

   XawToggleChangeRadioGroup(goBack,directionRadioGroup);
   XawToggleChangeRadioGroup(directionRadioGroup,goBack);


   setSearchDirection(FORWARD);
   setWhereToSearch(IN_HEADER);


   cancelButton = XtVaCreateManagedWidget(
				    "cancelSearch",
				    commandWidgetClass,
				    form,
				    XtNfromVert,directionRadioGroup,
				    XtNlabel,"Cancel",
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );


  XtAddCallback(cancelButton,XtNcallback,CBcancelSearch,NULL);


  searchButton = XtVaCreateManagedWidget(
				    "doTheSearch",
				    commandWidgetClass,
				    form,
				    XtNfromVert,directionRadioGroup,
				    XtNfromHoriz,cancelButton,
				    XtNlabel,"Search",
				    XtNshapeStyle,XmuShapeEllipse,
				    NULL
				   );


  XtAddCallback(searchButton,XtNcallback,CBdoSearch,NULL);


}


/*
 * popupSearchForm() - pop ups the search form for entry.
 *
 */

void popupSearchForm(where)
  Widget where;
{
  Position x,y;
  
  centerCoords(where,&x,&y);

  XtVaSetValues(searchPopup,
		XtNx,x,
		XtNy,y,
		NULL
	       );

  XtPopup(searchPopup,XtGrabExclusive);
  
}
 


/*
 * searchText() - returns the text in the search request window.
 *		     
 *
 */

char *searchText()
{
  char *text;

  XtVaGetValues(searchTextWidget,"string",&text,NULL);

  return text;
  
}


/*
 * doSearch() - searches from the current to the direction in the header
 *		or in the body. Returns TRUE if found something, FALSE else
 *		Direction should be PREVCARD or NEXTCARD
 *		Where     should be IN_HEADER or IN_BODY
 */

 
int doSearch(wanted)
  char *wanted;
{
  int result = FALSE;
  char *matchAgainst;
  
  while (!result && setCurrentCard(searchDirection()))
  {
    if (searchWhere() == IN_BODY)
    {
     result  = (strstr(bodyCurrentCard(),wanted) != NULL);
    }
    else
    {
     result  = (strstr(headerCurrentCard(),wanted) != NULL);
    }
  }

  return result;
}
